function [LLF, h, likelihoods] = skt_garchlikelihood(parameters , data , p , q, stdEstimate, T)
% PURPOSE:
%     Likelihood for fattailed garch estimation
% 
% USAGE:
%     [LLF, h, likelihoods] = fattailed_garchlikelihood(parameters , data , p , q, errortype, stdEstimate, T)
% 
% INPUTS:
%     parameters:   A vector of GARCH process aprams of the form [constant, arch, garch]
%     data:         A set of zero mean residuals
%     p:            The lag order length for ARCH
%     q:            The lag order length for GARCH
%     m:            The max of p and q
%     error:        SKEWED-T
%     stdEstimate:  The std deviation of the data
%     T:             Length of data
% 
% OUTPUTS:
%     LLF:          Minus 1 times the log likelihood
%     h:            Time series of conditional volatilities
%     likelihoods   Time series of likelihoods
% 
% COMMENTS:
%     This is a helper function for garchinmean
% 
% 
% Author: Kevin Sheppard
% kevin.sheppard@economics.ox.ac.uk
% Revision: 2    Date: 12/31/2001
% 



[r,c]=size(parameters);
if c>r
    parameters=parameters';
end
constp=parameters(1);
archp=parameters(2:p+1);
garchp=parameters(p+2:p+q+1);
shape=parameters(p+q+2);
nu = parameters(length(parameters));
parameters = parameters(1:p+q+1);
parameters(find(parameters <= 0)) = realmin;
if isempty(q)
    m=p;
else
    m  =  max(p,q);   
end

h=garchcore(data,parameters,stdEstimate^2,p,q,m,T);
t = (m + 1):T; %leave out the conditioning m obs
h=h(t);
data=data(t);
z=data./sqrt(h);
likelihoods=log(dskt(z,0,1,shape,nu));
LLF=-sum(likelihoods);

end





