
%% Simulate data from the SV model
% clear; clc;
% addpath(genpath('MISC_fn'));
% seed=123456;
% rng(seed);
% T=5000; 
% k=0.95; sigv=0.2;
% d=0; 
% [y,x]=gen_SV( T, d, k, sigv );
% figure;
% subplot(2,1,1); plot(y); title('Simulated y');
% subplot(2,1,2); plot(x); title('Simulated log variance');
% save('SVdata.mat');


%% ABC loop
% This code makes use of the GARCH and MFE Toolboxes 
% developed by Kevin Shephard.
% This ABC loop estimates the SV model using the GARCH-n auxiliary model.
% The options to the 'tarch' and 'tarch_scores' functions can be altered
% to change the specification of the auxiliary GARCH model.
clear; clc;
addpath(genpath('MISC_fn'));
addpath(genpath('Ucsd_garch'));
addpath(genpath('MFEToolbox'));
load('SVdata.mat','y','T','d','k','sigv');

%Change the input options here for alternative specifications of GARCH
%models.
T=1000;
y=y(1:T);
[parameters, LL, ht, robustSE, VCV, dat_score, diagnostics] = tarch(y, 1, 0, 1, 'NORMAL',2);

%number of ABC draws per run
%as per Frazier et al.
% eps=50/(T.^(3/2));
% P=500;
% reps=round(P/eps);

%for a fixed number of reps
eps=0.01; %proportions to keep
reps=50000; %ABC reps
P=round(eps*reps);

%Set the upper and lower bounds of the parameter space
lk=0.5; uk=0.99;
ls=0.05; us=0.5;
kdraw=lk+rand(reps,1)*(uk-lk);  %draw from prior k~unif(lk,uk)
sdraw=ls+rand(reps,1)*(us-ls); %draw from prior sigv~unif(ls,us)
%Storage for score distance
GARCH_dist=zeros(reps,1);
tic;
parfor i=1:reps
   z=gen_SV( T, d, kdraw(i), sdraw(i));
   %evaluate the score using the simulated z and the parameters estimated
   %from the data
   [sim_score]=tarch_scores(z, 1, 0, 1, 1, 2, parameters); 
   score_dist=mean(sim_score)-mean(dat_score);
   GARCH_dist(i)=(score_dist/VCV)*score_dist';
end
abc_time=toc;

%sorting the draws based on score distance
theta_GARCH=[kdraw,sdraw,GARCH_dist];
theta_GARCH_sorted=sortrows(theta_GARCH,3);

%select the top "P" chosen draws;
kg=linspace(lk,uk,100);
sigg=linspace(ls,us,100);
draw_GARCH=theta_GARCH_sorted(1:P,1:2);
[kdraw_kern_GARCH]=Gael_kernel(kg,draw_GARCH(:,1));
[sdraw_kern_GARCH]=Gael_kernel(sigg,draw_GARCH(:,2));

%save output
save('SV_ABCaux2_T1000.mat');

figure;
subplot(2,1,1); plot(kg,[kdraw_kern_GARCH]);
title('ABC - with GARCH-n');
xlabel('\k');
legend('GARCH-n','Location','northwest');
subplot(2,1,2); plot(sigg,[sdraw_kern_GARCH]);
title('ABC - with GARCH-n auxiliary models');
xlabel('\sigma');
legend('GARCH-n','Location','northwest');

%% using the ABC draws to obtain predictive density of obs T+1
%use particle filter here!
eps_t=normrnd(0,1,1,P); %use same random numbers for all predictions for comparability
ygrid=linspace(-10,10,100);

pred_GARCH=zeros(length(ygrid),1);

for i=1:P
   predbhati=predY_BPF_SVOU( ygrid, y, d, draw_GARCH(i,1), draw_GARCH(i,2));
   pred_GARCH=pred_GARCH+predbhati;
end
pred_GARCH=pred_GARCH./P;

yy=load('SVdata.mat','y');

figure;
plot(ygrid,[pred_GARCH]);
hold on;
line([yy.y(T+1) yy.y(T+1)],get(gca,'YLim'),'Color',[1 0 0]);
legend('GARCH-n');
title('Predictive density of y_{T+1}');

save('SV_ABCaux2_T1000.mat');


%% load in MCMC results for comparison

load('SV_MCMC_k_sigv.mat', 's_post_kern', 'k_post_kern', 'predy_MCMC');

figure;
subplot(2,1,1); plot(kg,[k_post_kern,kdraw_kern_GARCH]);
title('ABC - with GARCH-n');
xlabel('k');
legend('MCMC','ABC with GARCH-n','Location','northwest');
subplot(2,1,2); plot(sigg,[s_post_kern,sdraw_kern_GARCH]);
title('ABC - with GARCH-n auxiliary models');
xlabel('\sigma');
legend('MCMC','ABC with GARCH-n','Location','northwest');

figure;
plot(ygrid,[predy_MCMC,pred_GARCH]);
hold on;
line([yy.y(T+1) yy.y(T+1)],get(gca,'YLim'),'Color',[1 0 0]);
legend('MCMC','ABC with GARCH-n','Location','northwest');
title('Predictive density of y_{T+1}');

save('SV_ABCaux2_T1000.mat');

%% Forward simulation for comparison

pred_GARCH_sim=zeros(length(ygrid),1);

for i=1:P
   predbhati=predY_sim_SVOU( ygrid, T, d, draw_GARCH(i,1), draw_GARCH(i,2));
   pred_GARCH_sim=pred_GARCH_sim+predbhati;
end
pred_GARCH_sim=pred_GARCH_sim./P;

figure;
plot(ygrid,[predy_MCMC,pred_GARCH,pred_GARCH_sim]);
hold on;
line([yy.y(T+1) yy.y(T+1)],get(gca,'YLim'),'Color',[1 0 0]);
legend('MCMC','ABC with GARCH-n - Particle Filter', 'ABC with GARCH-n - Forward Simulation','Location','northwest');
title('Predictive density of y_{T+1}');


save('SV_ABCaux2_T1000.mat');
